@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/extended.c,v 1.23.2.5 2004/01/01 18:16:29 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/extended.c,v 1.23.2.6 2004/10/20 20:06:51 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2004 The OpenLDAP Foundation.
@@ -353,7 +353,13 @@ ldap_parse_intermediate (
 
 	tag = ber_peek_tag( ber, &len );
 
-	if( tag == LDAP_TAG_EXOP_RES_OID ) {
+	/*
+	 * NOTE: accept intermediate and extended response tag values
+	 * as older versions of slapd(8) incorrectly used extended
+	 * response tags.
+	 * Should be removed when 2.2 is moved to Historic.
+	 */
+	if( tag == LDAP_TAG_IM_RES_OID || tag == LDAP_TAG_EXOP_RES_OID ) {
 		/* we have a resoid */
 		if( ber_scanf( ber, "a", &resoid ) == LBER_ERROR ) {
 			ld->ld_errno = LDAP_DECODING_ERROR;
@@ -364,7 +370,7 @@ ldap_parse_intermediate (
 		tag = ber_peek_tag( ber, &len );
 	}
 
-	if( tag == LDAP_TAG_EXOP_RES_VALUE ) {
+	if( tag == LDAP_TAG_IM_RES_VALUE || tag == LDAP_TAG_EXOP_RES_VALUE ) {
 		/* we have a resdata */
 		if( ber_scanf( ber, "O", &resdata ) == LBER_ERROR ) {
 			ld->ld_errno = LDAP_DECODING_ERROR;
