@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/schema.c,v 1.61.2.2 2004/01/01 18:16:30 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/schema.c,v 1.61.2.4 2004/09/23 23:17:09 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2004 The OpenLDAP Foundation.
@@ -1183,8 +1183,8 @@ parse_qdescrs(const char **sp, int *code)
 					}
 					res = res1;
 				}
-				res[pos] = sval;
-				pos++;
+				res[pos++] = sval;
+				res[pos] = NULL;
 				parse_whsp(sp);
 			} else {
 				LDAP_VFREE(res);
@@ -1193,7 +1193,6 @@ parse_qdescrs(const char **sp, int *code)
 				return(NULL);
 			}
 		}
-		res[pos] = NULL;
 		parse_whsp(sp);
 		return(res);
 	} else if ( kind == TK_QDESCR ) {
@@ -1311,8 +1310,8 @@ parse_oids(const char **sp, int *code, const int allow_quoted)
 		kind = get_token(sp,&sval);
 		if ( kind == TK_BAREWORD ||
 		     ( allow_quoted && kind == TK_QDSTRING ) ) {
-			res[pos] = sval;
-			pos++;
+			res[pos++] = sval;
+			res[pos] = NULL;
 		} else {
 			*code = LDAP_SCHERR_UNEXPTOKEN;
 			LDAP_FREE(sval);
@@ -1341,8 +1340,8 @@ parse_oids(const char **sp, int *code, const int allow_quoted)
 						}
 						res = res1;
 					}
-					res[pos] = sval;
-					pos++;
+					res[pos++] = sval;
+					res[pos] = NULL;
 				} else {
 					*code = LDAP_SCHERR_UNEXPTOKEN;
 					LDAP_FREE(sval);
@@ -1357,7 +1356,6 @@ parse_oids(const char **sp, int *code, const int allow_quoted)
 				return NULL;
 			}
 		}
-		res[pos] = NULL;
 		parse_whsp(sp);
 		return(res);
 	} else if ( kind == TK_BAREWORD ||
@@ -1497,7 +1495,7 @@ ldap_str2syntax( LDAP_CONST char * s,
 		case TK_RIGHTPAREN:
 			return syn;
 		case TK_BAREWORD:
-			if ( !strcmp(sval,"NAME") ) {
+			if ( !strcasecmp(sval,"NAME") ) {
 				LDAP_FREE(sval);
 				if ( seen_name ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -1514,7 +1512,7 @@ ldap_str2syntax( LDAP_CONST char * s,
 					ldap_syntax_free(syn);
 					return NULL;
 				}
-			} else if ( !strcmp(sval,"DESC") ) {
+			} else if ( !strcasecmp(sval,"DESC") ) {
 				LDAP_FREE(sval);
 				if ( seen_desc ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -1627,11 +1625,11 @@ ldap_str2matchingrule( LDAP_CONST char * s,
 			ss = savepos;
 			kind = get_token(&ss,&sval);
 			if ( kind == TK_BAREWORD ) {
-				if ( !strcmp(sval, "NAME") ||
-				     !strcmp(sval, "DESC") ||
-				     !strcmp(sval, "OBSOLETE") ||
-				     !strcmp(sval, "SYNTAX") ||
-				     !strncmp(sval, "X-", 2) ) {
+				if ( !strcasecmp(sval, "NAME") ||
+				     !strcasecmp(sval, "DESC") ||
+				     !strcasecmp(sval, "OBSOLETE") ||
+				     !strcasecmp(sval, "SYNTAX") ||
+				     !strncasecmp(sval, "X-", 2) ) {
 					/* Missing OID, backtrack */
 					ss = savepos;
 				} else {
@@ -1667,7 +1665,7 @@ ldap_str2matchingrule( LDAP_CONST char * s,
 			}
 			return mr;
 		case TK_BAREWORD:
-			if ( !strcmp(sval,"NAME") ) {
+			if ( !strcasecmp(sval,"NAME") ) {
 				LDAP_FREE(sval);
 				if ( seen_name ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -1684,7 +1682,7 @@ ldap_str2matchingrule( LDAP_CONST char * s,
 					ldap_matchingrule_free(mr);
 					return NULL;
 				}
-			} else if ( !strcmp(sval,"DESC") ) {
+			} else if ( !strcasecmp(sval,"DESC") ) {
 				LDAP_FREE(sval);
 				if ( seen_desc ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -1704,7 +1702,7 @@ ldap_str2matchingrule( LDAP_CONST char * s,
 				}
 				mr->mr_desc = sval;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"OBSOLETE") ) {
+			} else if ( !strcasecmp(sval,"OBSOLETE") ) {
 				LDAP_FREE(sval);
 				if ( seen_obsolete ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -1715,7 +1713,7 @@ ldap_str2matchingrule( LDAP_CONST char * s,
 				seen_obsolete = 1;
 				mr->mr_obsolete = LDAP_SCHEMA_YES;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"SYNTAX") ) {
+			} else if ( !strcasecmp(sval,"SYNTAX") ) {
 				LDAP_FREE(sval);
 				if ( seen_syntax ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -1826,11 +1824,11 @@ ldap_str2matchingruleuse( LDAP_CONST char * s,
 			ss = savepos;
 			kind = get_token(&ss,&sval);
 			if ( kind == TK_BAREWORD ) {
-				if ( !strcmp(sval, "NAME") ||
-				     !strcmp(sval, "DESC") ||
-				     !strcmp(sval, "OBSOLETE") ||
-				     !strcmp(sval, "APPLIES") ||
-				     !strncmp(sval, "X-", 2) ) {
+				if ( !strcasecmp(sval, "NAME") ||
+				     !strcasecmp(sval, "DESC") ||
+				     !strcasecmp(sval, "OBSOLETE") ||
+				     !strcasecmp(sval, "APPLIES") ||
+				     !strncasecmp(sval, "X-", 2) ) {
 					/* Missing OID, backtrack */
 					ss = savepos;
 				} else {
@@ -1866,7 +1864,7 @@ ldap_str2matchingruleuse( LDAP_CONST char * s,
 			}
 			return mru;
 		case TK_BAREWORD:
-			if ( !strcmp(sval,"NAME") ) {
+			if ( !strcasecmp(sval,"NAME") ) {
 				LDAP_FREE(sval);
 				if ( seen_name ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -1883,7 +1881,7 @@ ldap_str2matchingruleuse( LDAP_CONST char * s,
 					ldap_matchingruleuse_free(mru);
 					return NULL;
 				}
-			} else if ( !strcmp(sval,"DESC") ) {
+			} else if ( !strcasecmp(sval,"DESC") ) {
 				LDAP_FREE(sval);
 				if ( seen_desc ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -1903,7 +1901,7 @@ ldap_str2matchingruleuse( LDAP_CONST char * s,
 				}
 				mru->mru_desc = sval;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"OBSOLETE") ) {
+			} else if ( !strcasecmp(sval,"OBSOLETE") ) {
 				LDAP_FREE(sval);
 				if ( seen_obsolete ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -1914,7 +1912,7 @@ ldap_str2matchingruleuse( LDAP_CONST char * s,
 				seen_obsolete = 1;
 				mru->mru_obsolete = LDAP_SCHEMA_YES;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"APPLIES") ) {
+			} else if ( !strcasecmp(sval,"APPLIES") ) {
 				LDAP_FREE(sval);
 				if ( seen_applies ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2042,19 +2040,19 @@ ldap_str2attributetype( LDAP_CONST char * s,
 			ss = savepos;
 			kind = get_token(&ss,&sval);
 			if ( kind == TK_BAREWORD ) {
-				if ( !strcmp(sval, "NAME") ||
-				     !strcmp(sval, "DESC") ||
-				     !strcmp(sval, "OBSOLETE") ||
-				     !strcmp(sval, "SUP") ||
-				     !strcmp(sval, "EQUALITY") ||
-				     !strcmp(sval, "ORDERING") ||
-				     !strcmp(sval, "SUBSTR") ||
-				     !strcmp(sval, "SYNTAX") ||
-				     !strcmp(sval, "SINGLE-VALUE") ||
-				     !strcmp(sval, "COLLECTIVE") ||
-				     !strcmp(sval, "NO-USER-MODIFICATION") ||
-				     !strcmp(sval, "USAGE") ||
-				     !strncmp(sval, "X-", 2) ) {
+				if ( !strcasecmp(sval, "NAME") ||
+				     !strcasecmp(sval, "DESC") ||
+				     !strcasecmp(sval, "OBSOLETE") ||
+				     !strcasecmp(sval, "SUP") ||
+				     !strcasecmp(sval, "EQUALITY") ||
+				     !strcasecmp(sval, "ORDERING") ||
+				     !strcasecmp(sval, "SUBSTR") ||
+				     !strcasecmp(sval, "SYNTAX") ||
+				     !strcasecmp(sval, "SINGLE-VALUE") ||
+				     !strcasecmp(sval, "COLLECTIVE") ||
+				     !strcasecmp(sval, "NO-USER-MODIFICATION") ||
+				     !strcasecmp(sval, "USAGE") ||
+				     !strncasecmp(sval, "X-", 2) ) {
 					/* Missing OID, backtrack */
 					ss = savepos;
 				} else if ( flags
@@ -2090,7 +2088,7 @@ ldap_str2attributetype( LDAP_CONST char * s,
 		case TK_RIGHTPAREN:
 			return at;
 		case TK_BAREWORD:
-			if ( !strcmp(sval,"NAME") ) {
+			if ( !strcasecmp(sval,"NAME") ) {
 				LDAP_FREE(sval);
 				if ( seen_name ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2107,7 +2105,7 @@ ldap_str2attributetype( LDAP_CONST char * s,
 					ldap_attributetype_free(at);
 					return NULL;
 				}
-			} else if ( !strcmp(sval,"DESC") ) {
+			} else if ( !strcasecmp(sval,"DESC") ) {
 				LDAP_FREE(sval);
 				if ( seen_desc ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2127,7 +2125,7 @@ ldap_str2attributetype( LDAP_CONST char * s,
 				}
 				at->at_desc = sval;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"OBSOLETE") ) {
+			} else if ( !strcasecmp(sval,"OBSOLETE") ) {
 				LDAP_FREE(sval);
 				if ( seen_obsolete ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2138,7 +2136,7 @@ ldap_str2attributetype( LDAP_CONST char * s,
 				seen_obsolete = 1;
 				at->at_obsolete = LDAP_SCHEMA_YES;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"SUP") ) {
+			} else if ( !strcasecmp(sval,"SUP") ) {
 				LDAP_FREE(sval);
 				if ( seen_sup ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2153,7 +2151,7 @@ ldap_str2attributetype( LDAP_CONST char * s,
 					ldap_attributetype_free(at);
 					return NULL;
 				}
-			} else if ( !strcmp(sval,"EQUALITY") ) {
+			} else if ( !strcasecmp(sval,"EQUALITY") ) {
 				LDAP_FREE(sval);
 				if ( seen_equality ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2168,7 +2166,7 @@ ldap_str2attributetype( LDAP_CONST char * s,
 					ldap_attributetype_free(at);
 					return NULL;
 				}
-			} else if ( !strcmp(sval,"ORDERING") ) {
+			} else if ( !strcasecmp(sval,"ORDERING") ) {
 				LDAP_FREE(sval);
 				if ( seen_ordering ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2183,7 +2181,7 @@ ldap_str2attributetype( LDAP_CONST char * s,
 					ldap_attributetype_free(at);
 					return NULL;
 				}
-			} else if ( !strcmp(sval,"SUBSTR") ) {
+			} else if ( !strcasecmp(sval,"SUBSTR") ) {
 				LDAP_FREE(sval);
 				if ( seen_substr ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2198,7 +2196,7 @@ ldap_str2attributetype( LDAP_CONST char * s,
 					ldap_attributetype_free(at);
 					return NULL;
 				}
-			} else if ( !strcmp(sval,"SYNTAX") ) {
+			} else if ( !strcasecmp(sval,"SYNTAX") ) {
 				LDAP_FREE(sval);
 				if ( seen_syntax ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2242,7 +2240,7 @@ ldap_str2attributetype( LDAP_CONST char * s,
 				    }
 				}
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"SINGLE-VALUE") ) {
+			} else if ( !strcasecmp(sval,"SINGLE-VALUE") ) {
 				LDAP_FREE(sval);
 				if ( at->at_single_value ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2252,7 +2250,7 @@ ldap_str2attributetype( LDAP_CONST char * s,
 				}
 				at->at_single_value = LDAP_SCHEMA_YES;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"COLLECTIVE") ) {
+			} else if ( !strcasecmp(sval,"COLLECTIVE") ) {
 				LDAP_FREE(sval);
 				if ( at->at_collective ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2262,7 +2260,7 @@ ldap_str2attributetype( LDAP_CONST char * s,
 				}
 				at->at_collective = LDAP_SCHEMA_YES;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"NO-USER-MODIFICATION") ) {
+			} else if ( !strcasecmp(sval,"NO-USER-MODIFICATION") ) {
 				LDAP_FREE(sval);
 				if ( at->at_no_user_mod ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2272,7 +2270,7 @@ ldap_str2attributetype( LDAP_CONST char * s,
 				}
 				at->at_no_user_mod = LDAP_SCHEMA_YES;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"USAGE") ) {
+			} else if ( !strcasecmp(sval,"USAGE") ) {
 				LDAP_FREE(sval);
 				if ( seen_usage ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2417,16 +2415,16 @@ ldap_str2objectclass( LDAP_CONST char * s,
 			ss = savepos;
 			kind = get_token(&ss,&sval);
 			if ( kind == TK_BAREWORD ) {
-				if ( !strcmp(sval, "NAME") ||
-				     !strcmp(sval, "DESC") ||
-				     !strcmp(sval, "OBSOLETE") ||
-				     !strcmp(sval, "SUP") ||
-				     !strcmp(sval, "ABSTRACT") ||
-				     !strcmp(sval, "STRUCTURAL") ||
-				     !strcmp(sval, "AUXILIARY") ||
-				     !strcmp(sval, "MUST") ||
-				     !strcmp(sval, "MAY") ||
-				     !strncmp(sval, "X-", 2) ) {
+				if ( !strcasecmp(sval, "NAME") ||
+				     !strcasecmp(sval, "DESC") ||
+				     !strcasecmp(sval, "OBSOLETE") ||
+				     !strcasecmp(sval, "SUP") ||
+				     !strcasecmp(sval, "ABSTRACT") ||
+				     !strcasecmp(sval, "STRUCTURAL") ||
+				     !strcasecmp(sval, "AUXILIARY") ||
+				     !strcasecmp(sval, "MUST") ||
+				     !strcasecmp(sval, "MAY") ||
+				     !strncasecmp(sval, "X-", 2) ) {
 					/* Missing OID, backtrack */
 					ss = savepos;
 				} else if ( flags &
@@ -2462,7 +2460,7 @@ ldap_str2objectclass( LDAP_CONST char * s,
 		case TK_RIGHTPAREN:
 			return oc;
 		case TK_BAREWORD:
-			if ( !strcmp(sval,"NAME") ) {
+			if ( !strcasecmp(sval,"NAME") ) {
 				LDAP_FREE(sval);
 				if ( seen_name ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2479,7 +2477,7 @@ ldap_str2objectclass( LDAP_CONST char * s,
 					ldap_objectclass_free(oc);
 					return NULL;
 				}
-			} else if ( !strcmp(sval,"DESC") ) {
+			} else if ( !strcasecmp(sval,"DESC") ) {
 				LDAP_FREE(sval);
 				if ( seen_desc ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2499,7 +2497,7 @@ ldap_str2objectclass( LDAP_CONST char * s,
 				}
 				oc->oc_desc = sval;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"OBSOLETE") ) {
+			} else if ( !strcasecmp(sval,"OBSOLETE") ) {
 				LDAP_FREE(sval);
 				if ( seen_obsolete ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2510,7 +2508,7 @@ ldap_str2objectclass( LDAP_CONST char * s,
 				seen_obsolete = 1;
 				oc->oc_obsolete = LDAP_SCHEMA_YES;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"SUP") ) {
+			} else if ( !strcasecmp(sval,"SUP") ) {
 				LDAP_FREE(sval);
 				if ( seen_sup ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2527,7 +2525,7 @@ ldap_str2objectclass( LDAP_CONST char * s,
 					ldap_objectclass_free(oc);
 					return NULL;
 				}
-			} else if ( !strcmp(sval,"ABSTRACT") ) {
+			} else if ( !strcasecmp(sval,"ABSTRACT") ) {
 				LDAP_FREE(sval);
 				if ( seen_kind ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2538,7 +2536,7 @@ ldap_str2objectclass( LDAP_CONST char * s,
 				seen_kind = 1;
 				oc->oc_kind = LDAP_SCHEMA_ABSTRACT;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"STRUCTURAL") ) {
+			} else if ( !strcasecmp(sval,"STRUCTURAL") ) {
 				LDAP_FREE(sval);
 				if ( seen_kind ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2549,7 +2547,7 @@ ldap_str2objectclass( LDAP_CONST char * s,
 				seen_kind = 1;
 				oc->oc_kind = LDAP_SCHEMA_STRUCTURAL;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"AUXILIARY") ) {
+			} else if ( !strcasecmp(sval,"AUXILIARY") ) {
 				LDAP_FREE(sval);
 				if ( seen_kind ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2560,7 +2558,7 @@ ldap_str2objectclass( LDAP_CONST char * s,
 				seen_kind = 1;
 				oc->oc_kind = LDAP_SCHEMA_AUXILIARY;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"MUST") ) {
+			} else if ( !strcasecmp(sval,"MUST") ) {
 				LDAP_FREE(sval);
 				if ( seen_must ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2576,7 +2574,7 @@ ldap_str2objectclass( LDAP_CONST char * s,
 					return NULL;
 				}
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"MAY") ) {
+			} else if ( !strcasecmp(sval,"MAY") ) {
 				LDAP_FREE(sval);
 				if ( seen_may ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2689,9 +2687,36 @@ ldap_str2contentrule( LDAP_CONST char * s,
 	savepos = ss;
 	cr->cr_oid = ldap_int_parse_numericoid(&ss,code,0);
 	if ( !cr->cr_oid ) {
-		*errp = ss;
-		ldap_contentrule_free(cr);
-		return NULL;
+		if ( (flags & LDAP_SCHEMA_ALLOW_ALL) && (ss == savepos) ) {
+			/* Backtracking */
+			ss = savepos;
+			kind = get_token(&ss,&sval);
+			if ( kind == TK_BAREWORD ) {
+				if ( !strcasecmp(sval, "NAME") ||
+				     !strcasecmp(sval, "DESC") ||
+				     !strcasecmp(sval, "OBSOLETE") ||
+				     !strcasecmp(sval, "AUX") ||
+				     !strcasecmp(sval, "MUST") ||
+				     !strcasecmp(sval, "MAY") ||
+				     !strcasecmp(sval, "NOT") ||
+				     !strncasecmp(sval, "X-", 2) ) {
+					/* Missing OID, backtrack */
+					ss = savepos;
+				} else if ( flags &
+					LDAP_SCHEMA_ALLOW_OID_MACRO ) {
+					/* Non-numerical OID, ignore */
+					int len = ss-savepos;
+					cr->cr_oid = LDAP_MALLOC(len+1);
+					strncpy(cr->cr_oid, savepos, len);
+					cr->cr_oid[len] = 0;
+				}
+			}
+			LDAP_FREE(sval);
+		} else {
+			*errp = ss;
+			ldap_contentrule_free(cr);
+			return NULL;
+		}
 	}
 	parse_whsp(&ss);
 
@@ -2710,7 +2735,7 @@ ldap_str2contentrule( LDAP_CONST char * s,
 		case TK_RIGHTPAREN:
 			return cr;
 		case TK_BAREWORD:
-			if ( !strcmp(sval,"NAME") ) {
+			if ( !strcasecmp(sval,"NAME") ) {
 				LDAP_FREE(sval);
 				if ( seen_name ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2727,7 +2752,7 @@ ldap_str2contentrule( LDAP_CONST char * s,
 					ldap_contentrule_free(cr);
 					return NULL;
 				}
-			} else if ( !strcmp(sval,"DESC") ) {
+			} else if ( !strcasecmp(sval,"DESC") ) {
 				LDAP_FREE(sval);
 				if ( seen_desc ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2747,7 +2772,7 @@ ldap_str2contentrule( LDAP_CONST char * s,
 				}
 				cr->cr_desc = sval;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"OBSOLETE") ) {
+			} else if ( !strcasecmp(sval,"OBSOLETE") ) {
 				LDAP_FREE(sval);
 				if ( seen_obsolete ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2758,7 +2783,7 @@ ldap_str2contentrule( LDAP_CONST char * s,
 				seen_obsolete = 1;
 				cr->cr_obsolete = LDAP_SCHEMA_YES;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"AUX") ) {
+			} else if ( !strcasecmp(sval,"AUX") ) {
 				LDAP_FREE(sval);
 				if ( seen_aux ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2774,7 +2799,7 @@ ldap_str2contentrule( LDAP_CONST char * s,
 					return NULL;
 				}
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"MUST") ) {
+			} else if ( !strcasecmp(sval,"MUST") ) {
 				LDAP_FREE(sval);
 				if ( seen_must ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2790,7 +2815,7 @@ ldap_str2contentrule( LDAP_CONST char * s,
 					return NULL;
 				}
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"MAY") ) {
+			} else if ( !strcasecmp(sval,"MAY") ) {
 				LDAP_FREE(sval);
 				if ( seen_may ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2806,7 +2831,7 @@ ldap_str2contentrule( LDAP_CONST char * s,
 					return NULL;
 				}
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"NOT") ) {
+			} else if ( !strcasecmp(sval,"NOT") ) {
 				LDAP_FREE(sval);
 				if ( seen_not ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2939,7 +2964,7 @@ ldap_str2structurerule( LDAP_CONST char * s,
 			}
 			return sr;
 		case TK_BAREWORD:
-			if ( !strcmp(sval,"NAME") ) {
+			if ( !strcasecmp(sval,"NAME") ) {
 				LDAP_FREE(sval);
 				if ( seen_name ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2956,7 +2981,7 @@ ldap_str2structurerule( LDAP_CONST char * s,
 					ldap_structurerule_free(sr);
 					return NULL;
 				}
-			} else if ( !strcmp(sval,"DESC") ) {
+			} else if ( !strcasecmp(sval,"DESC") ) {
 				LDAP_FREE(sval);
 				if ( seen_desc ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2976,7 +3001,7 @@ ldap_str2structurerule( LDAP_CONST char * s,
 				}
 				sr->sr_desc = sval;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"OBSOLETE") ) {
+			} else if ( !strcasecmp(sval,"OBSOLETE") ) {
 				LDAP_FREE(sval);
 				if ( seen_obsolete ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -2987,7 +3012,7 @@ ldap_str2structurerule( LDAP_CONST char * s,
 				seen_obsolete = 1;
 				sr->sr_obsolete = LDAP_SCHEMA_YES;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"FORM") ) {
+			} else if ( !strcasecmp(sval,"FORM") ) {
 				LDAP_FREE(sval);
 				if ( seen_nameform ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -3128,7 +3153,7 @@ ldap_str2nameform( LDAP_CONST char * s,
 			}
 			return nf;
 		case TK_BAREWORD:
-			if ( !strcmp(sval,"NAME") ) {
+			if ( !strcasecmp(sval,"NAME") ) {
 				LDAP_FREE(sval);
 				if ( seen_name ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -3145,7 +3170,7 @@ ldap_str2nameform( LDAP_CONST char * s,
 					ldap_nameform_free(nf);
 					return NULL;
 				}
-			} else if ( !strcmp(sval,"DESC") ) {
+			} else if ( !strcasecmp(sval,"DESC") ) {
 				LDAP_FREE(sval);
 				if ( seen_desc ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -3165,7 +3190,7 @@ ldap_str2nameform( LDAP_CONST char * s,
 				}
 				nf->nf_desc = sval;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"OBSOLETE") ) {
+			} else if ( !strcasecmp(sval,"OBSOLETE") ) {
 				LDAP_FREE(sval);
 				if ( seen_obsolete ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -3176,7 +3201,7 @@ ldap_str2nameform( LDAP_CONST char * s,
 				seen_obsolete = 1;
 				nf->nf_obsolete = LDAP_SCHEMA_YES;
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"MUST") ) {
+			} else if ( !strcasecmp(sval,"MUST") ) {
 				LDAP_FREE(sval);
 				if ( seen_must ) {
 					*code = LDAP_SCHERR_DUPOPT;
@@ -3192,7 +3217,7 @@ ldap_str2nameform( LDAP_CONST char * s,
 					return NULL;
 				}
 				parse_whsp(&ss);
-			} else if ( !strcmp(sval,"MAY") ) {
+			} else if ( !strcasecmp(sval,"MAY") ) {
 				LDAP_FREE(sval);
 				if ( seen_may ) {
 					*code = LDAP_SCHERR_DUPOPT;
