@@ -1,5 +1,5 @@
 /* cache.c - routines to maintain an in-core cache of entries */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/cache.c,v 1.38.2.11 2004/07/16 19:51:43 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/cache.c,v 1.38.2.12 2004/10/09 04:26:50 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2000-2004 The OpenLDAP Foundation.
@@ -565,7 +565,8 @@ bdb_cache_find_info(
 	struct bdb_info *bdb,
 	ID id )
 {
-	EntryInfo ei, *ei2;
+	EntryInfo ei = { 0 };
+	EntryInfo *ei2;
 
 	ei.bei_id = id;
 
@@ -595,7 +596,7 @@ bdb_cache_find_id(
 	struct bdb_info *bdb = (struct bdb_info *) op->o_bd->be_private;
 	Entry	*ep = NULL;
 	int	rc = 0;
-	EntryInfo ei;
+	EntryInfo ei = { 0 };
 
 	ei.bei_id = id;
 
