@@ -1,5 +1,5 @@
 /* search.c - search operation */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/search.c,v 1.136.2.18 2004/06/16 18:19:57 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/search.c,v 1.136.2.22 2004/10/16 17:49:32 jongchoi Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2000-2004 The OpenLDAP Foundation.
@@ -149,7 +149,7 @@ static int search_aliases(
 	ID cursora, ida, cursoro, ido, *subscop2;
 	Entry *matched, *a;
 	EntryInfo *ei;
-	struct berval bv_alias = { sizeof("alias")-1, "alias" };
+	struct berval bv_alias = BER_BVC( "alias" );
 	AttributeAssertion aa_alias;
 	Filter	af;
 	DB_LOCK locka, lockr;
@@ -215,9 +215,12 @@ static int search_aliases(
 			ida = bdb_idl_next(curscop, &cursora))
 		{
 			ei = NULL;
+retry1:
 			rs->sr_err = bdb_cache_find_id(op, NULL,
 				ida, &ei, 0, locker, &lockr );
 			if (rs->sr_err != LDAP_SUCCESS) {
+				if ( rs->sr_err == DB_LOCK_DEADLOCK ||
+					rs->sr_err == DB_LOCK_NOTGRANTED ) goto retry1;
 				continue;
 			}
 			a = ei->bei_e;
@@ -281,9 +284,15 @@ nextido:
 		 * Set the name so that the scope's IDL can be retrieved.
 		 */
 		ei = NULL;
+sameido:
 		rs->sr_err = bdb_cache_find_id(op, NULL, ido, &ei,
 			0, locker, &locka );
-		if ( rs->sr_err != LDAP_SUCCESS ) goto nextido;
+		if ( rs->sr_err != LDAP_SUCCESS ) {
+			if ( rs->sr_err == DB_LOCK_DEADLOCK ||
+				rs->sr_err == DB_LOCK_NOTGRANTED )
+				goto sameido;
+			goto nextido;
+		}
 		e = ei->bei_e;
 	}
 	return rs->sr_err;
@@ -326,13 +335,39 @@ int
 bdb_abandon( Operation *op, SlapReply *rs )
 {
 	Operation	*ps;
+	void		*saved_tmpmemctx;
 
 	ps = bdb_drop_psearch( op, op->oq_abandon.rs_msgid );
 	if ( ps ) {
-		if ( ps->o_tmpmemctx ) {
-			sl_mem_destroy( NULL, ps->o_tmpmemctx );
+		if ( ps->o_savmemctx ) {
+			ps->o_tmpmemctx = ps->o_savmemctx;
+			ps->o_tmpmfuncs = &sl_mfuncs;
+			ber_set_option(ps->o_ber, LBER_OPT_BER_MEMCTX, &ps->o_savmemctx);
 		}
+		saved_tmpmemctx = ps->o_tmpmemctx;
+
+		if (!BER_BVISNULL(&ps->o_req_dn)) {
+			slap_sl_free( ps->o_req_dn.bv_val, ps->o_tmpmemctx );
+		}
+		if (!BER_BVISNULL(&ps->o_req_ndn)) {
+			slap_sl_free( ps->o_req_ndn.bv_val, ps->o_tmpmemctx );
+		}
+		if (!BER_BVISNULL(&ps->ors_filterstr)) {
+			ps->o_tmpfree(ps->ors_filterstr.bv_val, ps->o_tmpmemctx);
+		}
+		if (ps->ors_filter != NULL) {
+			filter_free_x(ps, ps->ors_filter);
+		}
+		if (ps->ors_attrs != NULL) {
+			ps->o_tmpfree(ps->ors_attrs, ps->o_tmpmemctx);
+		}
+
 		slap_op_free ( ps );
+
+		if ( saved_tmpmemctx ) {
+			sl_mem_destroy( NULL, saved_tmpmemctx );
+		}
+
 		return LDAP_SUCCESS;
 	}
 	return LDAP_UNAVAILABLE;
@@ -342,15 +377,41 @@ int
 bdb_cancel( Operation *op, SlapReply *rs )
 {
 	Operation	*ps;
+	void		*saved_tmpmemctx;
 
 	ps = bdb_drop_psearch( op, op->oq_cancel.rs_msgid );
 	if ( ps ) {
+		if ( ps->o_savmemctx ) {
+			ps->o_tmpmemctx = ps->o_savmemctx;
+			ps->o_tmpmfuncs = &sl_mfuncs;
+			ber_set_option(ps->o_ber, LBER_OPT_BER_MEMCTX, &ps->o_savmemctx);
+		}
+		saved_tmpmemctx = ps->o_tmpmemctx;
 		rs->sr_err = LDAP_CANCELLED;
 		send_ldap_result( ps, rs );
-		if ( ps->o_tmpmemctx ) {
-			sl_mem_destroy( NULL, ps->o_tmpmemctx );
+
+		if (!BER_BVISNULL(&ps->o_req_dn)) {
+			slap_sl_free( ps->o_req_dn.bv_val, ps->o_tmpmemctx );
 		}
+		if (!BER_BVISNULL(&ps->o_req_ndn)) {
+			slap_sl_free( ps->o_req_ndn.bv_val, ps->o_tmpmemctx );
+		}
+		if (!BER_BVISNULL(&ps->ors_filterstr)) {
+			ps->o_tmpfree(ps->ors_filterstr.bv_val, ps->o_tmpmemctx);
+		}
+		if (ps->ors_filter != NULL) {
+			filter_free_x(ps, ps->ors_filter);
+		}
+		if (ps->ors_attrs != NULL) {
+			ps->o_tmpfree(ps->ors_attrs, ps->o_tmpmemctx);
+		}
+
 		slap_op_free ( ps );
+
+		if ( saved_tmpmemctx ) {
+			sl_mem_destroy( NULL, saved_tmpmemctx );
+		}
+
 		return LDAP_SUCCESS;
 	}
 	return LDAP_UNAVAILABLE;
@@ -361,6 +422,15 @@ int bdb_search( Operation *op, SlapReply *rs )
 	return bdb_do_search( op, rs, op, NULL, 0 );
 }
 
+int bdb_psearch( Operation *op, SlapReply *rs, Operation *sop,
+	Entry *ps_e, int ps_type )
+{
+	int	rc;
+
+	rc = bdb_do_search( op, rs, sop, ps_e, ps_type );
+	return rc;
+}
+
 /* For persistent searches, op is the currently executing operation,
  * sop is the persistent search. For regular searches, sop = op.
  */
@@ -408,14 +478,20 @@ bdb_do_search( Operation *op, SlapReply *rs, Operation *sop,
 	BerVarray	syncUUID_set = NULL;
 	int			syncUUID_set_cnt = 0;
 
+	struct	bdb_op_info	*opinfo = NULL;
+	DB_TXN			*ltid = NULL;
+
+	void *memctx_null = NULL;
+
 #ifdef NEW_LOGGING
 	LDAP_LOG( OPERATION, ENTRY, "bdb_search\n", 0, 0, 0 );
 #else
-	Debug( LDAP_DEBUG_TRACE, "=> bdb_search\n",
-		0, 0, 0);
+	Debug( LDAP_DEBUG_TRACE, "=> bdb_search\n", 0, 0, 0);
 #endif
 	attrs = sop->oq_search.rs_attrs;
 
+	opinfo = (struct bdb_op_info *) op->o_private;
+
 	if ( !IS_PSEARCH && sop->o_sync_mode & SLAP_SYNC_REFRESH_AND_PERSIST ) {
 		struct slap_session_entry *sent;
 		if ( sop->o_sync_state.sid >= 0 ) {
@@ -428,6 +504,15 @@ bdb_do_search( Operation *op, SlapReply *rs, Operation *sop,
 		}
 	}
 
+	if ( !IS_PSEARCH && sop->o_sync_mode & SLAP_SYNC_PERSIST ) {
+		if ( sop->o_tmpmemctx ) {
+			sop->o_savmemctx = sop->o_tmpmemctx;
+			sop->o_tmpmemctx = NULL;
+			sop->o_tmpmfuncs = &ch_mfuncs;
+			ber_set_option( op->o_ber, LBER_OPT_BER_MEMCTX, &memctx_null );
+		}
+	}
+
 	/* psearch needs to be registered before refresh begins */
 	/* psearch and refresh transmission is serialized in send_ldap_ber() */
 	if ( !IS_PSEARCH && sop->o_sync_mode & SLAP_SYNC_PERSIST ) {
@@ -479,8 +564,7 @@ bdb_do_search( Operation *op, SlapReply *rs, Operation *sop,
 	null_attr.an_desc = NULL;
 	null_attr.an_oc = NULL;
 	null_attr.an_oc_exclude = 0;
-	null_attr.an_name.bv_len = 0;
-	null_attr.an_name.bv_val = NULL;
+	BER_BVZERO( &null_attr.an_name );
 
 	for( num_ctrls = 0; num_ctrls < SLAP_MAX_RESPONSE_CONTROLS; num_ctrls++ ) {
 		ctrls[num_ctrls] = NULL;
@@ -492,8 +576,7 @@ bdb_do_search( Operation *op, SlapReply *rs, Operation *sop,
 		attrs[0].an_desc = NULL;
 		attrs[0].an_oc = NULL;
 		attrs[0].an_oc_exclude = 0;
-		attrs[0].an_name.bv_len = 0;
-		attrs[0].an_name.bv_val = NULL;
+		BER_BVZERO( &attrs[0].an_name );
 	}
 
 	manageDSAit = get_manageDSAit( sop );
@@ -509,14 +592,19 @@ bdb_do_search( Operation *op, SlapReply *rs, Operation *sop,
 		}
 	}
 
-	rs->sr_err = LOCK_ID( bdb->bi_dbenv, &locker );
+	if ( opinfo && opinfo->boi_txn ) {
+		ltid = opinfo->boi_txn;
+		locker = TXN_ID( ltid );
+	} else {
+		rs->sr_err = LOCK_ID( bdb->bi_dbenv, &locker );
 
-	switch(rs->sr_err) {
-	case 0:
-		break;
-	default:
-		send_ldap_error( sop, rs, LDAP_OTHER, "internal error" );
-		return rs->sr_err;
+		switch(rs->sr_err) {
+		case 0:
+			break;
+		default:
+			send_ldap_error( sop, rs, LDAP_OTHER, "internal error" );
+			return rs->sr_err;
+		}
 	}
 
 	if ( sop->o_req_ndn.bv_len == 0 ) {
@@ -525,14 +613,14 @@ bdb_do_search( Operation *op, SlapReply *rs, Operation *sop,
 		ei_root.bei_parent = &ei_root;
 		e_root.e_private = &ei_root;
 		e_root.e_id = 0;
-		e_root.e_nname.bv_val="";
-		e_root.e_name.bv_val="";
+		BER_BVSTR( &e_root.e_nname, "" );
+		BER_BVSTR( &e_root.e_name, "" );
 		ei = &ei_root;
 		rs->sr_err = LDAP_SUCCESS;
 	} else {
 dn2entry_retry:
 		/* get entry with reader lock */
-		rs->sr_err = bdb_dn2entry( op, NULL, &sop->o_req_ndn, &ei,
+		rs->sr_err = bdb_dn2entry( op, ltid, &sop->o_req_ndn, &ei,
 			1, locker, &lock );
 	}
 
@@ -545,14 +633,16 @@ dn2entry_retry:
 		break;
 	case LDAP_BUSY:
 		send_ldap_error( sop, rs, LDAP_BUSY, "ldap server busy" );
-		LOCK_ID_FREE (bdb->bi_dbenv, locker );
+		if ( !opinfo )
+			LOCK_ID_FREE (bdb->bi_dbenv, locker );
 		return LDAP_BUSY;
 	case DB_LOCK_DEADLOCK:
 	case DB_LOCK_NOTGRANTED:
 		goto dn2entry_retry;
 	default:
 		send_ldap_error( sop, rs, LDAP_OTHER, "internal error" );
-		LOCK_ID_FREE (bdb->bi_dbenv, locker );
+		if ( !opinfo )
+			LOCK_ID_FREE (bdb->bi_dbenv, locker );
 		return rs->sr_err;
 	}
 
@@ -592,7 +682,8 @@ dn2entry_retry:
 		rs->sr_matched = matched_dn.bv_val;
 		send_ldap_result( sop, rs );
 
-		LOCK_ID_FREE (bdb->bi_dbenv, locker );
+		if ( !opinfo )
+			LOCK_ID_FREE (bdb->bi_dbenv, locker );
 		if ( rs->sr_ref ) {
 			ber_bvarray_free( rs->sr_ref );
 			rs->sr_ref = NULL;
@@ -634,7 +725,8 @@ dn2entry_retry:
 		rs->sr_matched = matched_dn.bv_val;
 		send_ldap_result( sop, rs );
 
-		LOCK_ID_FREE (bdb->bi_dbenv, locker );
+		if ( !opinfo )
+			LOCK_ID_FREE (bdb->bi_dbenv, locker );
 		ber_bvarray_free( rs->sr_ref );
 		rs->sr_ref = NULL;
 		ber_memfree( matched_dn.bv_val );
@@ -895,7 +987,7 @@ loop_begin:
 id2entry_retry:
 			/* get the entry with reader lock */
 			ei = NULL;
-			rs->sr_err = bdb_cache_find_id( op, NULL,
+			rs->sr_err = bdb_cache_find_id( op, ltid,
 				id, &ei, 0, locker, &lock );
 
 			if (rs->sr_err == LDAP_BUSY) {
@@ -1468,7 +1560,8 @@ done:
 		bdb_cache_return_entry_r( bdb->bi_dbenv, &bdb->bi_cache, e, &lock );
 	}
 
-	LOCK_ID_FREE( bdb->bi_dbenv, locker );
+	if ( !opinfo )
+		LOCK_ID_FREE( bdb->bi_dbenv, locker );
 
 	ber_bvfree( search_context_csn );
 
@@ -1615,7 +1708,7 @@ static int search_candidates(
 	{
 		if( !get_manageDSAit(op) && !get_domainScope(op) ) {
 			/* match referral objects */
-			struct berval bv_ref = { sizeof("referral")-1, "referral" };
+			struct berval bv_ref = BER_BVC( "referral" );
 			rf.f_choice = LDAP_FILTER_EQUALITY;
 			rf.f_ava = &aa_ref;
 			rf.f_av_desc = slap_schema.si_ad_objectClass;
@@ -1639,7 +1732,7 @@ static int search_candidates(
 
 #ifdef BDB_SUBENTRIES
 	if( get_subentries_visibility( op ) ) {
-		struct berval bv_subentry = { sizeof("SUBENTRY")-1, "SUBENTRY" };
+		struct berval bv_subentry = BER_BVC( "SUBENTRY" );
 		sf.f_choice = LDAP_FILTER_EQUALITY;
 		sf.f_ava = &aa_subentry;
 		sf.f_av_desc = slap_schema.si_ad_objectClass;
@@ -1709,20 +1802,20 @@ send_paged_response(
 	LDAPControl	ctrl, *ctrls[2];
 	BerElementBuffer berbuf;
 	BerElement	*ber = (BerElement *)&berbuf;
-	struct berval	cookie = BER_BVC( "" );
 	PagedResultsCookie respcookie;
+	struct berval cookie;
 
 #ifdef NEW_LOGGING
 	LDAP_LOG ( OPERATION, ENTRY,
-		"send_paged_response: lastid: (0x%08lx) "
-		"nentries: (0x%081x)\n", 
+		"send_paged_response: lastid=0x%08lx nentries=%d\n", 
 		lastid ? *lastid : 0, rs->sr_nentries, NULL );
 #else
-	Debug(LDAP_DEBUG_ARGS, "send_paged_response: lastid: (0x%08lx) "
-		"nentries: (0x%081x)\n", lastid ? *lastid : 0, rs->sr_nentries, NULL );
+	Debug(LDAP_DEBUG_ARGS,
+		"send_paged_response: lastid=0x%08lx nentries=%d\n", 
+		lastid ? *lastid : 0, rs->sr_nentries, NULL );
 #endif
 
-	ctrl.ldctl_value.bv_val = NULL;
+	BER_BVZERO( &ctrl.ldctl_value );
 	ctrls[0] = &ctrl;
 	ctrls[1] = NULL;
 
@@ -1735,17 +1828,15 @@ send_paged_response(
 
 	} else {
 		respcookie = ( PagedResultsCookie )0;
+		BER_BVSTR( &cookie, "" );
 	}
 
 	op->o_conn->c_pagedresults_state.ps_cookie = respcookie;
 	op->o_conn->c_pagedresults_state.ps_count =
 		op->o_pagedresults_state.ps_count + rs->sr_nentries;
 
-	/*
-	 * FIXME: we should consider sending an estimate of the entries
-	 * left, after appropriate security check is done
-	 */
-	ber_printf( ber, "{iO}", tentries, &cookie ); 
+	/* return size of 0 -- no estimate */
+	ber_printf( ber, "{iO}", 0, &cookie ); 
 
 	if ( ber_flatten2( ber, &ctrls[0]->ldctl_value, 0 ) == -1 ) {
 		goto done;
