@@ -1,5 +1,5 @@
 /* modify.c - ldbm backend modify routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/modify.c,v 1.100.2.5 2004/01/01 18:16:37 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/modify.c,v 1.100.2.7 2004/09/23 22:32:13 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2004 The OpenLDAP Foundation.
@@ -37,8 +37,7 @@ int ldbm_modify_internal(
     Entry	*e,
 	const char **text,
 	char *textbuf,
-	size_t textlen
-)
+	size_t textlen )
 {
 	int rc = LDAP_SUCCESS;
 	Modification	*mod;
@@ -47,12 +46,19 @@ int ldbm_modify_internal(
 	Attribute 	*ap;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG( BACK_LDBM, ENTRY,  "ldbm_modify_internal: %s\n", e->e_name.bv_val, 0, 0 );
+	LDAP_LOG( BACK_LDBM, ENTRY,
+		"ldbm_modify_internal: %s\n",
+		e->e_name.bv_val,
+		get_permissiveModify(op) ? " (permissive)" : "",
+		0 );
 #else
-	Debug(LDAP_DEBUG_TRACE, "ldbm_modify_internal: %s\n", e->e_name.bv_val, 0, 0);
+	Debug(LDAP_DEBUG_TRACE,
+		"ldbm_modify_internal: %s\n",
+		e->e_name.bv_val,
+		get_permissiveModify(op) ? " (permissive)" : "",
+		0 );
 #endif
 
-
 	if ( !acl_check_modlist( op, e, modlist )) {
 		return LDAP_INSUFFICIENT_ACCESS;
 	}
@@ -66,13 +72,15 @@ int ldbm_modify_internal(
 		switch ( mod->sm_op ) {
 		case LDAP_MOD_ADD:
 #ifdef NEW_LOGGING
-			LDAP_LOG( BACK_LDBM, DETAIL1, "ldbm_modify_internal: add\n", 0, 0, 0);
+			LDAP_LOG( BACK_LDBM, DETAIL1,
+				"ldbm_modify_internal: add\n", 0, 0, 0);
 #else
-			Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: add\n", 0, 0, 0);
+			Debug(LDAP_DEBUG_ARGS,
+				"ldbm_modify_internal: add\n", 0, 0, 0);
 #endif
 
 			rc = modify_add_values( e, mod, get_permissiveModify( op ),
-						text, textbuf, textlen );
+				text, textbuf, textlen );
 			if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
 				LDAP_LOG( BACK_LDBM, INFO, 
@@ -86,13 +94,15 @@ int ldbm_modify_internal(
 
 		case LDAP_MOD_DELETE:
 #ifdef NEW_LOGGING
-			LDAP_LOG( BACK_LDBM, DETAIL1, "ldbm_modify_internal: delete\n", 0,0,0);
+			LDAP_LOG( BACK_LDBM, DETAIL1,
+				"ldbm_modify_internal: delete\n", 0,0,0);
 #else
-			Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: delete\n", 0, 0, 0);
+			Debug(LDAP_DEBUG_ARGS,
+				"ldbm_modify_internal: delete\n", 0, 0, 0);
 #endif
 
 			rc = modify_delete_values( e, mod, get_permissiveModify( op ),
-							text, textbuf, textlen );
+				text, textbuf, textlen );
 			assert( rc != LDAP_TYPE_OR_VALUE_EXISTS );
 			if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
@@ -107,13 +117,15 @@ int ldbm_modify_internal(
 
 		case LDAP_MOD_REPLACE:
 #ifdef NEW_LOGGING
-			LDAP_LOG( BACK_LDBM, DETAIL1, "ldbm_modify_internal:  replace\n",0,0,0);
+			LDAP_LOG( BACK_LDBM, DETAIL1,
+				"ldbm_modify_internal:  replace\n",0,0,0);
 #else
-			Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: replace\n", 0, 0, 0);
+			Debug(LDAP_DEBUG_ARGS,
+				"ldbm_modify_internal: replace\n", 0, 0, 0);
 #endif
 
 			rc = modify_replace_values( e, mod, get_permissiveModify( op ),
-							text, textbuf, textlen );
+				text, textbuf, textlen );
 			if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
 				LDAP_LOG( BACK_LDBM, INFO, 
@@ -152,7 +164,8 @@ int ldbm_modify_internal(
 			LDAP_LOG( BACK_LDBM, DETAIL1, 
 				"ldbm_modify_internal: softadd\n", 0, 0, 0 );
 #else
-			Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: softadd\n", 0, 0, 0);
+			Debug(LDAP_DEBUG_ARGS,
+				"ldbm_modify_internal: softadd\n", 0, 0, 0);
 #endif
 
 			/* Avoid problems in index_add_mods()
@@ -161,7 +174,7 @@ int ldbm_modify_internal(
 			mod->sm_op = LDAP_MOD_ADD;
 
 			rc = modify_add_values( e, mod, get_permissiveModify( op ),
-						text, textbuf, textlen );
+				text, textbuf, textlen );
 			mod->sm_op = SLAP_MOD_SOFTADD;
 			if ( rc == LDAP_TYPE_OR_VALUE_EXISTS ) {
 				rc = LDAP_SUCCESS;
@@ -170,7 +183,7 @@ int ldbm_modify_internal(
 			if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
 				LDAP_LOG( BACK_LDBM, INFO, 
-					   "ldbm_modify_internal: failed %d (%s)\n", rc, *text, 0 );
+				   "ldbm_modify_internal: failed %d (%s)\n", rc, *text, 0 );
 #else
 				Debug(LDAP_DEBUG_ARGS, "ldbm_modify_internal: %d %s\n",
 					rc, *text, 0);
@@ -255,8 +268,8 @@ int ldbm_modify_internal(
 					0, 0, 0	);
 #else
 				Debug( LDAP_DEBUG_ANY,
-				       "Attribute index delete failure",
-			               0, 0, 0 );
+					"ldbm_modify_internal: Attribute index delete failure\n",
+					0, 0, 0 );
 #endif
 				goto exit;
 			}
@@ -277,8 +290,8 @@ int ldbm_modify_internal(
 					0, 0, 0 );
 #else
 				Debug( LDAP_DEBUG_ANY,
-				       "Attribute index add failure",
-			               0, 0, 0 );
+					"ldbm_modify_internal: Attribute index add failure\n",
+					0, 0, 0 );
 #endif
 				goto exit;
 			}
@@ -326,6 +339,7 @@ ldbm_back_modify(
 
 	/* FIXME: dn2entry() should return non-glue entry */
 	if (( e == NULL ) || ( !manageDSAit && e && is_entry_glue( e ))) {
+		BerVarray deref = NULL;
 		if ( matched != NULL ) {
 			rs->sr_matched = ch_strdup( matched->e_dn );
 			rs->sr_ref = is_entry_referral( matched )
@@ -333,7 +347,6 @@ ldbm_back_modify(
 				: NULL;
 			cache_return_entry_r( &li->li_cache, matched );
 		} else {
-			BerVarray deref = NULL;
 			if ( !LDAP_STAILQ_EMPTY( &op->o_bd->be_syncinfo )) {
 				syncinfo_t *si;
 				LDAP_STAILQ_FOREACH( si, &op->o_bd->be_syncinfo, si_next ) {
@@ -345,7 +358,7 @@ ldbm_back_modify(
 				deref = default_referral;
 			}
 			rs->sr_ref = referral_rewrite( deref, NULL, &op->o_req_dn,
-							LDAP_SCOPE_DEFAULT );
+						LDAP_SCOPE_DEFAULT );
 		}
 
 		ldap_pvt_thread_rdwr_wunlock(&li->li_giant_rwlock);
@@ -353,6 +366,9 @@ ldbm_back_modify(
 		send_ldap_result( op, rs );
 
 		if ( rs->sr_ref ) ber_bvarray_free( rs->sr_ref );
+		if ( deref != default_referral ) {
+			ber_bvarray_free( deref );
+		}
 		free( (char *)rs->sr_matched );
 
 		rs->sr_ref = NULL;
