@@ -1,5 +1,5 @@
 /* operation.c - deal with operation subsystem */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/operation.c,v 1.18.2.5 2004/03/18 00:56:29 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/operation.c,v 1.18.2.6 2004/04/12 18:20:14 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2001-2004 The OpenLDAP Foundation.
@@ -40,7 +40,7 @@ static struct berval
 		BER_BVC( "Search" ),
 		BER_BVC( "Abandon" ),
 		BER_BVC( "Extended" ),
-		{ 0, NULL }
+		BER_BVNULL
 	};
 
 int
