@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/entry-id.c,v 1.20.2.7 2004/03/18 22:43:35 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/entry-id.c,v 1.20.2.9 2004/09/24 14:09:15 ando Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1999-2004 The OpenLDAP Foundation.
@@ -38,12 +38,22 @@ backsql_free_entryID( backsql_entryID *id, int freeit )
 
 	assert( id );
 
-	next = id->next;
+	next = id->eid_next;
 
-	if ( id->dn.bv_val != NULL ) {
-		free( id->dn.bv_val );
+	if ( id->eid_dn.bv_val != NULL ) {
+		free( id->eid_dn.bv_val );
 	}
 
+#ifdef BACKSQL_ARBITRARY_KEY
+	if ( id->eid_id.bv_val ) {
+		free( id->eid_id.bv_val );
+	}
+
+	if ( id->eid_keyval.bv_val ) {
+		free( id->eid_keyval.bv_val );
+	}
+#endif /* BACKSQL_ARBITRARY_KEY */
+
 	if ( freeit ) {
 		free( id );
 	}
@@ -65,13 +75,18 @@ backsql_dn2id(
 
 	/* TimesTen */
 	char			upperdn[ BACKSQL_MAX_DN_LEN + 1 ];
-	char			*toBind;
+	struct berval		toBind;
 	int			i, j;
 
-	Debug( LDAP_DEBUG_TRACE, "==>backsql_dn2id(): dn='%s'\n", 
-			dn->bv_val, 0, 0 );
+	/*
+	 * NOTE: id can be NULL; in this case, the function
+	 * simply checks whether the DN can be successfully 
+	 * turned into an ID, returning LDAP_SUCCESS for
+	 * positive cases, or the most appropriate error
+	 */
 
-	assert( id );
+	Debug( LDAP_DEBUG_TRACE, "==>backsql_dn2id(): dn=\"%s\"%s\n", 
+			dn->bv_val, id == NULL ? " (no ID)" : "", 0 );
 
 	if ( dn->bv_len > BACKSQL_MAX_DN_LEN ) {
 		Debug( LDAP_DEBUG_TRACE, 
@@ -82,7 +97,7 @@ backsql_dn2id(
 	}
 	
 	/* begin TimesTen */
-	Debug(LDAP_DEBUG_TRACE, "id_query '%s'\n", bi->id_query, 0, 0);
+	Debug(LDAP_DEBUG_TRACE, "id_query \"%s\"\n", bi->id_query, 0, 0);
 	assert( bi->id_query );
  	rc = backsql_Prepare( dbh, &sth, bi->id_query, 0 );
 	if ( rc != SQL_SUCCESS ) {
@@ -106,29 +121,30 @@ backsql_dn2id(
 		upperdn[ i ] = '\0';
 		ldap_pvt_str2upper( upperdn );
 
-		Debug( LDAP_DEBUG_TRACE, "==>backsql_dn2id(): upperdn='%s'\n",
+		Debug( LDAP_DEBUG_TRACE, "==>backsql_dn2id(): upperdn=\"%s\"\n",
 				upperdn, 0, 0 );
-		toBind = upperdn;
+		ber_str2bv( upperdn, 0, 0, &toBind );
+
 	} else {
 		if ( BACKSQL_USE_REVERSE_DN( bi ) ) {
 			AC_MEMCPY( upperdn, dn->bv_val, dn->bv_len + 1 );
 			ldap_pvt_str2upper( upperdn );
 			Debug( LDAP_DEBUG_TRACE,
-				"==>backsql_dn2id(): upperdn='%s'\n",
+				"==>backsql_dn2id(): upperdn=\"%s\"\n",
 				upperdn, 0, 0 );
-			toBind = upperdn;
+			ber_str2bv( upperdn, 0, 0, &toBind );
 
 		} else {
-			toBind = dn->bv_val;
+			toBind = *dn;
 		}
 	}
 
-	rc = backsql_BindParamStr( sth, 1, toBind, BACKSQL_MAX_DN_LEN );
+	rc = backsql_BindParamStr( sth, 1, toBind.bv_val, BACKSQL_MAX_DN_LEN );
 	if ( rc != SQL_SUCCESS) {
 		/* end TimesTen */ 
 		Debug( LDAP_DEBUG_TRACE, "backsql_dn2id(): "
 			"error binding dn=\"%s\" parameter:\n", 
-			toBind, 0, 0 );
+			toBind.bv_val, 0, 0 );
 		backsql_PrintErrors( SQL_NULL_HENV, dbh, sth, rc );
 		SQLFreeStmt( sth, SQL_DROP );
 		return LDAP_OTHER;
@@ -138,7 +154,7 @@ backsql_dn2id(
 	if ( rc != SQL_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_dn2id(): "
 			"error executing query (\"%s\", \"%s\"):\n", 
-			bi->id_query, toBind, 0 );
+			bi->id_query, toBind.bv_val, 0 );
 		backsql_PrintErrors( SQL_NULL_HENV, dbh, sth, rc );
 		SQLFreeStmt( sth, SQL_DROP );
 		return LDAP_OTHER;
@@ -147,27 +163,32 @@ backsql_dn2id(
 	backsql_BindRowAsStrings( sth, &row );
 	rc = SQLFetch( sth );
 	if ( BACKSQL_SUCCESS( rc ) ) {
-		id->id = strtol( row.cols[ 0 ], NULL, 0 );
-		id->keyval = strtol( row.cols[ 1 ], NULL, 0 );
-		id->oc_id = strtol( row.cols[ 2 ], NULL, 0 );
-		ber_dupbv( &id->dn, dn );
-		id->next = NULL;
-
 		res = LDAP_SUCCESS;
+		Debug( LDAP_DEBUG_TRACE, "<==backsql_dn2id(): id=%s keyval=%s oc_id=%s\n",
+				row.cols[ 0 ], row.cols[ 1 ], row.cols[ 2 ] );
+
+		if ( id != NULL ) {
+#ifdef BACKSQL_ARBITRARY_KEY
+			ber_str2bv( row.cols[ 0 ], 0, 1, &id->eid_id );
+			ber_str2bv( row.cols[ 1 ], 0, 1, &id->eid_keyval );
+#else /* ! BACKSQL_ARBITRARY_KEY */
+			id->eid_id = strtol( row.cols[ 0 ], NULL, 0 );
+			id->eid_keyval = strtol( row.cols[ 1 ], NULL, 0 );
+#endif /* ! BACKSQL_ARBITRARY_KEY */
+			id->eid_oc_id = strtol( row.cols[ 2 ], NULL, 0 );
+
+			ber_dupbv( &id->eid_dn, dn );
+			id->eid_next = NULL;
+		}
 
 	} else {
 		res = LDAP_NO_SUCH_OBJECT;
+		Debug( LDAP_DEBUG_TRACE, "<==backsql_dn2id(): no match\n",
+				0, 0, 0 );
 	}
 	backsql_FreeRow( &row );
 
 	SQLFreeStmt( sth, SQL_DROP );
-	if ( res == LDAP_SUCCESS ) {
-		Debug( LDAP_DEBUG_TRACE, "<==backsql_dn2id(): id=%ld\n",
-				id->id, 0, 0 );
-	} else {
-		Debug( LDAP_DEBUG_TRACE, "<==backsql_dn2id(): no match\n",
-				0, 0, 0 );
-	}
 	return res;
 }
 
@@ -183,7 +204,7 @@ backsql_count_children(
 	RETCODE 		rc;
 	int			res = LDAP_SUCCESS;
 
-	Debug( LDAP_DEBUG_TRACE, "==>backsql_count_children(): dn='%s'\n", 
+	Debug( LDAP_DEBUG_TRACE, "==>backsql_count_children(): dn=\"%s\"\n", 
 			dn->bv_val, 0, 0 );
 
 	if ( dn->bv_len > BACKSQL_MAX_DN_LEN ) {
@@ -195,7 +216,7 @@ backsql_count_children(
 	}
 	
 	/* begin TimesTen */
-	Debug(LDAP_DEBUG_TRACE, "children id query '%s'\n", 
+	Debug(LDAP_DEBUG_TRACE, "children id query \"%s\"\n", 
 			bi->has_children_query, 0, 0);
 	assert( bi->has_children_query );
  	rc = backsql_Prepare( dbh, &sth, bi->has_children_query, 0 );
@@ -287,11 +308,18 @@ backsql_get_attr_vals( void *v_at, void *v_bsi )
 
 	assert( at );
 	assert( bsi );
- 
+
+#ifdef BACKSQL_ARBITRARY_KEY
 	Debug( LDAP_DEBUG_TRACE, "==>backsql_get_attr_vals(): "
-		"oc='%s' attr='%s' keyval=%ld\n",
+		"oc=\"%s\" attr=\"%s\" keyval=%s\n",
 		BACKSQL_OC_NAME( bsi->bsi_oc ), at->bam_ad->ad_cname.bv_val, 
-		bsi->bsi_c_eid->keyval );
+		bsi->bsi_c_eid->eid_keyval.bv_val );
+#else /* ! BACKSQL_ARBITRARY_KEY */
+	Debug( LDAP_DEBUG_TRACE, "==>backsql_get_attr_vals(): "
+		"oc=\"%s\" attr=\"%s\" keyval=%ld\n",
+		BACKSQL_OC_NAME( bsi->bsi_oc ), at->bam_ad->ad_cname.bv_val, 
+		bsi->bsi_c_eid->eid_keyval );
+#endif /* ! BACKSQL_ARBITRARY_KEY */
 
 	rc = backsql_Prepare( bsi->bsi_dbh, &sth, at->bam_query, 0 );
 	if ( rc != SQL_SUCCESS ) {
@@ -301,17 +329,34 @@ backsql_get_attr_vals( void *v_at, void *v_bsi )
 		return 1;
 	}
 
-	rc = backsql_BindParamID( sth, 1, &bsi->bsi_c_eid->keyval );
+#ifdef BACKSQL_ARBITRARY_KEY
+	rc = backsql_BindParamStr( sth, 1, bsi->bsi_c_eid->eid_keyval.bv_val,
+		       BACKSQL_MAX_KEY_LEN );
+#else /* ! BACKSQL_ARBITRARY_KEY */
+	rc = backsql_BindParamID( sth, 1, &bsi->bsi_c_eid->eid_keyval );
+#endif /* ! BACKSQL_ARBITRARY_KEY */
 	if ( rc != SQL_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_get_attr_values(): "
 			"error binding key value parameter\n", 0, 0, 0 );
 		return 1;
 	}
 
+#ifdef BACKSQL_TRACE
+#ifdef BACKSQL_ARBITRARY_KEY
+	Debug( LDAP_DEBUG_TRACE, "backsql_get_attr_values(): "
+		"query=\"%s\" keyval=%s\n", at->bam_query,
+		bsi->bsi_c_eid->eid_keyval.bv_val, 0 );
+#else /* !BACKSQL_ARBITRARY_KEY */
+	Debug( LDAP_DEBUG_TRACE, "backsql_get_attr_values(): "
+		"query=\"%s\" keyval=%d\n", at->bam_query,
+		bsi->bsi_c_eid->eid_keyval, 0 );
+#endif /* ! BACKSQL_ARBITRARY_KEY */
+#endif /* BACKSQL_TRACE */
+
 	rc = SQLExecute( sth );
 	if ( ! BACKSQL_SUCCESS( rc ) ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_get_attr_values(): "
-			"error executing attribute query '%s'\n",
+			"error executing attribute query \"%s\"\n",
 			at->bam_query, 0, 0 );
 		backsql_PrintErrors( bi->db_env, bsi->bsi_dbh, sth, rc );
 		SQLFreeStmt( sth, SQL_DROP );
@@ -345,7 +390,7 @@ backsql_get_attr_vals( void *v_at, void *v_bsi )
 					(int)row.col_prec[ i ], 0, 0 );
 			} else {
       				Debug( LDAP_DEBUG_TRACE, "NULL value "
-					"in this row for attribute '%s'\n",
+					"in this row for attribute \"%s\"\n",
 					row.col_names[ i ].bv_val, 0, 0 );
 #endif /* BACKSQL_TRACE */
 			}
@@ -363,8 +408,8 @@ backsql_get_attr_vals( void *v_at, void *v_bsi )
 	return 1;
 }
 
-Entry *
-backsql_id2entry( backsql_srch_info *bsi, Entry *e, backsql_entryID *eid )
+int
+backsql_id2entry( backsql_srch_info *bsi, backsql_entryID *eid )
 {
 	int			i;
 	int			rc;
@@ -372,32 +417,44 @@ backsql_id2entry( backsql_srch_info *bsi, Entry *e, backsql_entryID *eid )
 
 	Debug( LDAP_DEBUG_TRACE, "==>backsql_id2entry()\n", 0, 0, 0 );
 
-	rc = dnPrettyNormal( NULL, &eid->dn, &e->e_name, &e->e_nname,
+	assert( bsi->bsi_e );
+
+	memset( bsi->bsi_e, 0, sizeof( Entry ) );
+
+	rc = dnPrettyNormal( NULL, &eid->eid_dn,
+			&bsi->bsi_e->e_name, &bsi->bsi_e->e_nname,
 			bsi->bsi_op->o_tmpmemctx );
 	if ( rc != LDAP_SUCCESS ) {
-		return NULL;
+		return rc;
 	}
 
-	bsi->bsi_oc = backsql_id2oc( bsi->bsi_op->o_bd->be_private, eid->oc_id );
-	bsi->bsi_e = e;
+	bsi->bsi_e->e_attrs = NULL;
+	bsi->bsi_e->e_private = NULL;
+
+	bsi->bsi_oc = backsql_id2oc( bsi->bsi_op->o_bd->be_private,
+			eid->eid_oc_id );
 	bsi->bsi_c_eid = eid;
-	e->e_attrs = NULL;
-	e->e_private = NULL;
- 
-	e->e_id = eid->id;
+
+#ifndef BACKSQL_ARBITRARY_KEY	
+	bsi->bsi_e->e_id = eid->eid_id;
+#endif /* ! BACKSQL_ARBITRARY_KEY */
  
+	rc = attr_merge_normalize_one( bsi->bsi_e, ad_oc,
+				&bsi->bsi_oc->bom_oc->soc_cname,
+				bsi->bsi_op->o_tmpmemctx );
+	if ( rc != LDAP_SUCCESS ) {
+		entry_clean( bsi->bsi_e );
+		return rc;
+	}
+
 	if ( bsi->bsi_attrs != NULL ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_id2entry(): "
 			"custom attribute list\n", 0, 0, 0 );
 		for ( i = 0; bsi->bsi_attrs[ i ].an_name.bv_val; i++ ) {
 			backsql_at_map_rec	**vat;
-			AttributeName		*attr = &bsi->bsi_attrs[ i ];
+			AttributeName		*an = &bsi->bsi_attrs[ i ];
 			int			j;
 
-			if ( attr->an_desc == ad_oc ) {
-				continue;
-			}
-
 			/* if one of the attributes listed here is
 			 * a subtype of another, it must be ignored,
 			 * because subtypes are already dealt with
@@ -410,18 +467,19 @@ backsql_id2entry( backsql_srch_info *bsi, Entry *e, backsql_entryID *eid )
 				}
 
 				/* skip subtypes */
-				if ( is_at_subtype( attr->an_desc->ad_type, bsi->bsi_attrs[ j ].an_desc->ad_type ) )
+				if ( is_at_subtype( an->an_desc->ad_type,
+							bsi->bsi_attrs[ j ].an_desc->ad_type ) )
 				{
 					goto next;
 				}
 			}
 
-			rc = backsql_supad2at( bsi->bsi_oc, attr->an_desc, &vat );
+			rc = backsql_supad2at( bsi->bsi_oc, an->an_desc, &vat );
 			if ( rc != 0 || vat == NULL ) {
 				Debug( LDAP_DEBUG_TRACE, "backsql_id2entry(): "
-						"attribute '%s' is not defined "
-						"for objectlass '%s'\n",
-						attr->an_name.bv_val, 
+						"attribute \"%s\" is not defined "
+						"for objectlass \"%s\"\n",
+						an->an_name.bv_val, 
 						BACKSQL_OC_NAME( bsi->bsi_oc ), 0 );
 				continue;
 			}
@@ -442,13 +500,6 @@ next:;
 				bsi, 0, AVL_INORDER );
 	}
 
-	if ( attr_merge_normalize_one( bsi->bsi_e, ad_oc,
-				&bsi->bsi_oc->bom_oc->soc_cname,
-				bsi->bsi_op->o_tmpmemctx ) ) {
-		entry_free( e );
-		return NULL;
-	}
-
 	if ( global_schemacheck ) {
 		const char	*text = NULL;
 		char		textbuf[ 1024 ];
@@ -463,8 +514,12 @@ next:;
 		rc = structural_class( bv, &soc, NULL, 
 				&text, textbuf, textlen );
 		if ( rc != LDAP_SUCCESS ) {
-			entry_free( e );
-			return NULL;
+      			Debug( LDAP_DEBUG_TRACE, "backsql_id2entry(%s): "
+				"structural_class() failed %d (%s)\n",
+				bsi->bsi_e->e_name.bv_val,
+				rc, text ? text : "" );
+			entry_clean( bsi->bsi_e );
+			return rc;
 		}
 
 		if ( ( bsi->bsi_flags | BSQL_SF_ALL_OPER )
@@ -473,15 +528,15 @@ next:;
 					slap_schema.si_ad_structuralObjectClass,
 					&soc, bsi->bsi_op->o_tmpmemctx );
 			if ( rc != LDAP_SUCCESS ) {
-				entry_free( e );
-				return NULL;
+				entry_clean( bsi->bsi_e );
+				return rc;
 			}
 		}
 	}
 
 	Debug( LDAP_DEBUG_TRACE, "<==backsql_id2entry()\n", 0, 0, 0 );
 
-	return e;
+	return LDAP_SUCCESS;
 }
 
 #endif /* SLAPD_SQL */
