@@ -1,5 +1,5 @@
 /* backend.c - routines for dealing with back-end databases */
-/* $OpenLDAP: pkg/ldap/servers/slapd/backend.c,v 1.206.2.22 2004/09/03 23:17:53 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/backend.c,v 1.206.2.24 2004/09/27 10:29:38 ando Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2004 The OpenLDAP Foundation.
@@ -143,7 +143,7 @@ static BackendInfo binfo[] = {
 	{"tcl",		tcl_back_initialize},
 #endif
 #if SLAPD_SQL == SLAPD_MOD_STATIC
-	{"sql",		sql_back_initialize},
+	{"sql",		backsql_initialize},
 #endif
 	/* for any private backend */
 #if SLAPD_PRIVATE == SLAPD_MOD_STATIC
@@ -520,11 +520,19 @@ int backend_destroy(void)
 {
 	int i;
 	BackendDB *bd;
+	syncinfo_t *si_entry;
 
 	ldap_pvt_thread_pool_destroy( &syncrepl_pool, 1 );
 
 	/* destroy each backend database */
 	for( i = 0, bd = backendDB; i < nBackendDB; i++, bd++ ) {
+
+		while ( !LDAP_STAILQ_EMPTY( &bd->be_syncinfo )) {
+			si_entry = LDAP_STAILQ_FIRST( &bd->be_syncinfo );
+			LDAP_STAILQ_REMOVE_HEAD( &bd->be_syncinfo, si_next );
+			syncinfo_free( si_entry );
+		}
+		
 		if ( bd->bd_info->bi_db_destroy ) {
 			bd->bd_info->bi_db_destroy( bd );
 		}
